package info.textgrid.middleware;

import info.textgrid.middleware.oaipmh.ListMetadataFormatsType;
import info.textgrid.middleware.oaipmh.MetadataFormatType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

import java.math.BigInteger;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Map;
import java.util.TimeZone;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.commons.logging.LogFactory;

/**
 * OAIPMHUtillities is responsible to implement all necessary functions to
 * implement the arguments of a request
 * 
 * @author Maximilian Brodhun: SUB
 * @version 1.0
 * @since 29.01.2014
 *
 */

public class OAIPMHUtillities {
	private static org.apache.commons.logging.Log log = LogFactory.getLog(OAIPMHImpl.class);
	/**
	 * Since textGrid just supports dc the only returned MetadataFormat is
	 * dublinCore
	 * 
	 * @return
	 */
	public MetadataFormatType setMetadataFormats() {

		MetadataFormatType tgDublinCore = new MetadataFormatType();
		tgDublinCore.setMetadataNamespace("http://www.openarchives.org/OAI/2.0/oai_dc/");
		tgDublinCore.setMetadataPrefix("oai:dc");
		tgDublinCore.setSchema("http://www.openarchives.org/OAI/2.0/oai_dc.xsd");

		return tgDublinCore;
	}

	/**
	 * producing the list of all metadata formats
	 * 
	 * @return tgRepMetadaFormats
	 */
	public ListMetadataFormatsType setMetadataFormatList() {

		ListMetadataFormatsType tgRepMetadataFormats = new ListMetadataFormatsType();
		MetadataFormatType tgDublinCore = setMetadataFormats();
		tgRepMetadataFormats.getMetadataFormat().add(tgDublinCore);

		return tgRepMetadataFormats;
	}

	/**
	 * calculating the current date and give it back as XMLGregorianCalendar
	 * 
	 * @return now
	 * @throws DatatypeConfigurationException
	 */
	public static XMLGregorianCalendar getXMLGregorianCalendarNow()
			throws DatatypeConfigurationException {

		GregorianCalendar gregorianCalendar = new GregorianCalendar();
		DatatypeFactory datatypeFactory = DatatypeFactory.newInstance();
		XMLGregorianCalendar now = datatypeFactory.newXMLGregorianCalendar(gregorianCalendar);

		return now;
	}

	/**
	 * Converting a given string representing a date value into the date format
	 * required for OAIPMH and give it back as XMLGregorianCalendar
	 * 
	 * @param originalDateTimeString
	 * @return xmlCal
	 * @throws ParseException
	 * @throws DatatypeConfigurationException
	 */
	public static XMLGregorianCalendar convertDateFormat(
			String originalDateTimeString) throws ParseException,
			DatatypeConfigurationException {

		SimpleDateFormat tgItemTime = new SimpleDateFormat(	"yyyy-MM-dd'T'HH:mm:ss.S");
		Date date = tgItemTime.parse(originalDateTimeString);
		SimpleDateFormat outFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
		outFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
		String output2 = outFormatter.format(date);
		XMLGregorianCalendar xmlCal = DatatypeFactory.newInstance().newXMLGregorianCalendar(output2);

		return xmlCal;
	}

	/**
	 * Converting a given string representing a date value into the date format
	 * required for OAIPMH and give it back as string
	 * 
	 * @param originalDateTimeString
	 * @return dateOutputAsString
	 * @throws ParseException
	 */
	public static String datestampAsString(String originalDateTimeString)
			throws ParseException {

		SimpleDateFormat tgItemTime = new SimpleDateFormat(
				"yyyy-MM-dd'T'HH:mm:ss.S");
		Date date = tgItemTime.parse(originalDateTimeString);
		SimpleDateFormat outFormatter = new SimpleDateFormat(
				"yyyy-MM-dd'T'HH:mm:ss'Z'");
		outFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
		String dateOutputAsString = outFormatter.format(date);

		return dateOutputAsString;
	}

	/**
	 * <p>
	 * Returns a resumptionToken for a search, if necessary.
	 * </p>
	 * 
	 * TODO Check if we really must put all these references in!
	 * 
	 * @param completeListSize
	 * @param resumptionToken
	 * @param cursorCollector
	 * @param scrollID
	 * @param searchResponseSize
	 * @param i
	 * @return A resumption token if applicable, null otherwise.
	 */
	public static ResumptionTokenType getResumptionToken(
			final long completeListSize, final String resumptionToken,
			Map<String, Integer> cursorCollector, final String scrollID,
			final int searchResponseSize, final int i) {
		
		log.info("Creating a ResumptionToken:\n " + 
					"CompleteListSize: " + completeListSize + "\n" +
					"ResumptionToken: " + resumptionToken + "\n" +
					"CursorCollector: " + cursorCollector + "\n" +
					"ScrollID: " + scrollID + "\n" +
					"SearchResponseSize: " + scrollID + "\n" + 
					"IterationStep: " + i);
		
		int cursor;

		// Three cases here:
		// 1. Complete list size is > 100 and a token already is existing: we
		// need to check hash map!
		// 2. Complete list size is > 100 and we have no token: we do need one!
		// 3. Complete list size is <= 100 (we do not need a token! do nothing!)

		if (completeListSize > 100) {
			ResumptionTokenType resTokenForResponse = new ResumptionTokenType();

			if (resumptionToken != null
					&& cursorCollector.containsKey(resumptionToken)) {
				cursor = cursorCollector.get(resumptionToken).intValue() + i;
				resTokenForResponse
						.setCursor(BigInteger.valueOf((long) cursor));
				cursorCollector.put(scrollID, cursor);
			} else {
				resTokenForResponse.setCursor(BigInteger
						.valueOf(searchResponseSize));
				cursorCollector.put(scrollID, searchResponseSize);
				cursor = 0;
			}

			// Set resumption token string if cursor is less then complete list
			// size: More objects can be delivered!
			if (cursor < completeListSize) {
				resTokenForResponse.setValue(scrollID);
			}

			// Remove resumption token string if cursor is bigger then or equal
			// complete list size: This is the last response, no more objects
			// are available!
			if (cursor >= completeListSize) {
				resTokenForResponse.setValue("");
			}

			// Set complete list size.
			resTokenForResponse.setCompleteListSize(BigInteger
					.valueOf(completeListSize));

			return resTokenForResponse;
		}

		return null;
	}

}
